/*
 * genscore
 *
 * This program generates the scoreboard in HTML from the report generated
 * from "java pc2.Report -all"
 *
 * To use this, type:
 *
 * genscore [-f] time teams root_name
 *
 * where:
 *
 *        -f: forces the scores to be updated even after score freezing time
 *      time: how often the generated web pages should refresh itself
 *     teams: how many teams per page
 * root_name: the root_name of the generated file names
 *
 * The following HTML files are generated:
 *
 * root_name.html: all the scores on one page
 * root_name0.html: top "teams" teams on one page
 * root_name1.html: next "teams" teams on one page
 *       ...
 *
 * It reads header.html to completely generate the web pages.  This file
 * should contain something like this:
 *
 * ----------------BEGIN FILE--------------------------
 * <head>
 * <title>...</title>
 * </head>
 * <body>
 * <h1>...</h1>
 * -----------------END FILE---------------------------
 *
 * The rest is filled in by this program.
 *
 * Note: we also allow simulated runs from a file "fake_runs", in the following
 * format:
 *
 * ----------------BEGIN FILE--------------------------
 * Name of fake team 1
 *    ...
 * Name of fake team n
 * #
 * team name|problem|minutes|accept|deleted
 *    ...
 * team name|problem|minutes|accept|deleted
 * #
 * -----------------END FILE---------------------------
 *
 * where problem is 1, 2, 3, ...
 *       accept is Y or N
 *       deleted is Y or N
 *
 * There is minimal/none error-checking on the fake_runs file.
 * Blank lines are ignored.
 *
 * Author: Howard Cheng
 *
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <ctype.h>

#define MAX_TEAMS 1000
#define MAX_NAME 1000
#define MAX_PROBS 50
#define MAX_FILENAME 10000

#define NOTSOLVED -1
#define JUDGING -2

#define FREEZE_FILE "freeze"
#define HEADER_FILE "header.html"
#define FAKE_FILE   "fake_runs"

typedef struct {
  char name[MAX_NAME+1];
  int attempt[MAX_PROBS];
  int score[MAX_PROBS];        /* -1 = not solved, -2 = judging */
  int solved, total, rank;
} Team;

Team team[MAX_TEAMS];

int num_teams, num_probs;
int length, elapsed, freeze;
int refresh, teams_per_page;
int force, running;
char root_name[MAX_FILENAME+1];
FILE *fake;

static int team_id(char *name);
static void get_times(void);
static void get_team_names(void);
static void get_num_probs(void);
static void get_runs(void);
static void compute_ranks(void);
static int team_cmp(const void *a, const void *b);
static void print_htmls(void);
static void print_html(char *filename, char *next, int start, int num);
static int is_blank(char *s);

int main(int argc, char *argv[])
{
  int i;

  /* parse arguments */
  if ((argc != 4 && argc != 5) || (argc == 5 && strcmp(argv[1], "-f"))) {
    fprintf(stderr, "Usage: %s [-f] time teams root_name\n", argv[0]);
    return -1;
  }
  force = (argc == 5);
  i = (argc == 5) ? 2 : 1;
  refresh = atoi(argv[i++]);
  teams_per_page = atoi(argv[i++]);
  strcpy(root_name, argv[i]);

  /* open fake file, if any */
  fake = fopen(FAKE_FILE, "r");

  /* get contest length, elapsed time, freeze time */
  get_times();

  /* get the team names */
  get_team_names();

  /* get number of problems */
  get_num_probs();

  /* parse the runs */
  get_runs();

  /* now sort them */
  qsort(team, num_teams, sizeof(Team), team_cmp);

  /* compute ranks */
  compute_ranks();

  /* produce web pages */
  print_htmls();

  if (fake) {
    fclose(fake);
  }

  return 0;
}

static int team_id(char *name)
{
  int i;
  for (i = 0; i < num_teams; i++) {
    if (!strcmp(name, team[i].name)) {
      return i;
    }
  }
  fprintf(stderr, "Team '%s' not found\n", name);
  exit(-1);
}

static void get_times(void)
{
  char buffer[10240];
  FILE *f;
  int h, m, s;

  /* get the freeze time from a file */
  if (!(f = fopen(FREEZE_FILE, "r"))) {
    fprintf(stderr, "Error: cannot open \"%s\"\n", FREEZE_FILE);
    exit(-1);
  }
  if (fscanf(f, "%d:%d:%d", &h, &m, &s) != 3) {
    fprintf(stderr, "Error: syntax error in freeze file.  Expecting hh:mm:ss\n");
    exit(-1);
  }
  freeze = h*60*60 + m*60 + s;
  fclose(f);

  /* get the length and elasped time from report */
  while (fgets(buffer, 10240, stdin) && !strstr(buffer, "Contest running"))
    ;
  running = (strstr(buffer, "true")) ? 1 : 0;

  while (fgets(buffer, 10240, stdin) && !strstr(buffer, "Contest elapsed"))
    ;
  if (sscanf(buffer, "Contest elapsed %d:%d:%d", &h, &m, &s) == 2) {
    s = m;
    m = h;
    h = 0;
  }
  elapsed = h*60*60 + m*60 + s;

  fgets(buffer, 10240, stdin);
  if (sscanf(buffer, "Contest length %d:%d:%d", &h, &m, &s) == 2) {
    s = m;
    m = h; 
    h = 0;
  }
  length = h*60*60 + m*60 + s;
}

static void get_team_names(void)
{
  char buffer[10240], *p;
  int l;

  while (fgets(buffer, 10240, stdin) && strcmp(buffer, "-- Teams --\n"))
    ;
  
  num_teams = 0;
  while (1) {
    if (!fgets(buffer, 10240, stdin)) {
      break;
    }
    l = strlen(buffer);
    if (buffer[l-1] == '\n') {
      buffer[l-1] = 0;
    }
    p = strchr(buffer, ':');
    if (!p) {
      break;
    }
    p = strchr(p+1, ':');
    if (!p) {
      break;
    }
    strcpy(team[num_teams++].name, p+1);
  }

  /* now get the fake teams */
  if (fake) {
    while (fgets(buffer, 10240, fake) && buffer[0] != '#') {
      l = strlen(buffer);
      if (buffer[l-1] == '\n') {
	buffer[l-1] = 0;
      }
      if (!is_blank(buffer)) {
	strcpy(team[num_teams++].name, buffer);
      }
    }
  }
}

static void get_num_probs(void)
{
  char buffer[10240];

  while (fgets(buffer, 10240, stdin) && strcmp(buffer, "-- Problems --\n"))
    ;
  
  num_probs = 0;
  while (fgets(buffer, 10240, stdin) && strcmp(buffer, "-- Judgements --\n")) {
    if (!isspace(buffer[0])) {
      num_probs++;
    }
  }
}

static void get_runs(void)
{
  char buffer[10240], buffer2[10240], *p, *s;
  int i, j;
  int team_index, prob, stime, deleted, accept;
  int done_real, done;

  for (i = 0; i < num_teams; i++) {
    team[i].solved = team[i].total = 0;
    for (j = 0; j < num_probs; j++) {
      team[i].attempt[j] = 0;
      team[i].score[j] = NOTSOLVED;
    }
  }

  while (fgets(buffer, 10240, stdin) && strcmp(buffer, "-- Runs --\n"))
    ;

  done_real = done = 0;
  while (!done) {
    if (!done_real) {
      if (!fgets(buffer, 10240, stdin) ||
	  strstr(buffer, "No runs to list/print")) {
	done_real = 1;
	continue;
      }

      /* first get the team name */
      p = strtok(buffer, "|\n");
      if (!p) {
	done_real = 1;
	continue;
      }
      p = strtok(0, "|\n");
      p = strtok(0, "|\n");
      p = strtok(0, "|\n");
      s = strchr(p, ':');
      team_index = team_id(s+1);
      
      /* now get problem */
      p = strtok(0, "|\n");
      sscanf(p, "prob %d", &prob);
      prob--;
      
      /* now get the time */
      p = strtok(0, "|\n");
      p = strtok(0, "|\n");
      p = strtok(0, "|\n");
      p = strtok(0, "|\n");
      p = strtok(0, "|\n");
      p = strtok(0, "|\n");
      stime = atoi(p);
            
      /* now see if it's deleted */
      p = strtok(0, "|\n");
      p = strtok(0, "|\n");
      sscanf(p, "del? %s", buffer2);
      deleted = (!strcmp(buffer2, "true")) ? 1 : 0;
      
      /* now get the judgement */
      p = strtok(0, "|\n");
      if (strcmp(p, "<null>")) {
	sscanf(p, "jci %d", &accept);
	accept = !accept;
	if (!accept) {
	  stime = NOTSOLVED;
	}
      } else {
	accept = 0;
	stime = JUDGING;
      }
    } else if (fake) {
      /* get the fake runs */
      if (!fgets(buffer, 10240, fake) || buffer[0] == '#') {
	done = 1;
	continue;
      }

      if (is_blank(buffer)) {
	continue;
      }

      p = strtok(buffer, "|\n");
      team_index = team_id(p);
      p = strtok(0, "|\n");
      prob = atoi(p) - 1;
      p = strtok(0, "|\n");
      stime = atoi(p);

      /* ignore if it hasn't really happened yet */
      if (stime * 60 > elapsed) {
	continue;
      }

      p = strtok(0, "|\n");
      accept = (*p == 'Y');
      p = strtok(0, "|\n");
      deleted = (*p == 'Y');
      if (!accept) {
	stime = NOTSOLVED;
      }
    } else {
      done = 1;
      continue;
    }
    
    /* if the run is past freezing time, ignore it */
    if (stime * 60 > freeze && !force) {
      continue;
    }

    /* don't do anything for deleted runs */
    if (deleted) {
      continue;
    }

    /*
     * count only up to the first correct run for each team/problem
     *
     */
    if (team[team_index].score[prob] < 0) {
      team[team_index].attempt[prob]++;
      team[team_index].score[prob] = stime;
      if (accept) {
	team[team_index].solved++;
	team[team_index].total += stime + 
	  20*(team[team_index].attempt[prob] - 1);
      }
    }
  }
}

static void compute_ranks(void)
{
  int i, r, ps, pt;

  r = 0;
  ps = -1; pt = 0;
  for (i = 0; i < num_teams; i++) {
    if (team[i].solved != ps || team[i].total != pt) {
      r = i+1;
    }
    team[i].rank = r;
    ps = team[i].solved;
    pt = team[i].total;
  }
}

static int team_cmp(const void *a, const void *b)
{
  Team *t1 = (Team *)a, *t2 = (Team *)b;

  if (t1->solved != t2->solved) {
    return t2->solved - t1->solved;
  } else if (t1->total != t2->total) {
    return t1->total - t2->total;
  } else {
    return strcmp(t1->name, t2->name);
  }
}

static void print_htmls(void)
{
  char name[MAX_FILENAME+1], next[MAX_FILENAME+1];
  int num_pages;
  int i, s;
  

  /* first, produce the complete listing */
  sprintf(name, "%s.html", root_name);
  print_html(name, 0, 0, num_teams);

  /* now generate those that cycles */
  num_pages = num_teams / teams_per_page;
  if (num_teams % teams_per_page) {
    num_pages++;
  }
  s = 0;
  for (i = 0; i < num_pages; i++) {
    sprintf(name, "%s%d.html", root_name, i);
    sprintf(next, "%s%d.html", root_name, (i+1 == num_pages) ? 0 : i+1);
    print_html(name, next, s, 
	       (s + teams_per_page >= num_teams) ? num_teams - s 
	       : teams_per_page);
    s += teams_per_page;
  }
}

static void print_html(char *name, char *next, int start, int num)
{
  FILE *f, *out;
  char buffer[10240];
  int c, i, j;
  struct tm *time_m;
  time_t tp;

  /* open the output file */
  if (!(out = fopen(name, "w"))) {
    fprintf(stderr, "Error: cannot write to \"%s\"\n", name);
    exit(-1);
  }

  /* first print the header */
  fprintf(out, "<html>\n");
  if (next) {
    fprintf(out, "<meta http-equiv=\"Refresh\" content=\"%d;URL=%s\">\n",
	    refresh, next);
  } else {
    fprintf(out, "<meta http-equiv=\"Refresh\" content=\"%d\">\n",
	    refresh);
  }

  if (!(f = fopen(HEADER_FILE, "r"))) {
    fprintf(stderr, "Error: cannot open \"%s\"\n", HEADER_FILE);
    exit(-1);
  }
  while ((c = fgetc(f)) != EOF) {
    fputc(c, out);
  }
  fclose(f);

  /* now print the times */
  time(&tp);
  time_m = localtime(&tp);
  strftime(buffer, 10240, "%a %b %d, %Y  %I:%M:%S %p  %Z", time_m);

  fprintf(out, "<h3>Last Updated: %s</h3>\n", buffer);
  fprintf(out, "<h3>Contest Length: %02d:%02d:%02d</h3>\n", length/3600, 
	 (length/60)%60, length%60);
  fprintf(out, "<h3>Contest Elapsed: %02d:%02d:%02d", elapsed/3600, 
	  (elapsed/60)%60, elapsed%60);
  if (elapsed > freeze && !force) {
    fprintf(out, " (scores frozen at %02d:%02d:%02d)", freeze/3600, 
	    (freeze/60)%60, freeze%60);
  }
  fprintf(out, "</h3>\n");
  
  fprintf(out, "<h3>The contest is %s.</h3>\n", 
	 (running) ? "running" : "not running");

  /* now print the table */
  fprintf(out, "<table border=1 width=100%%>\n");

  /* print header row */
  fprintf(out, "<th>Team</th>");
  for (i = 0; i < num_probs; i++) {
    fprintf(out, "<th>%c</th>", 'A'+i);
  }
  fprintf(out, "<th>Solved</th><th>Penalty</th>");
  fprintf(out, "<th>Rank</th></tr>\n");

  /* print the scores for each team */
  for (i = 0; i < num; i++) {
    c = start+i;

    /* team name */
    fprintf(out, "<tr><td align=left>%s</td>", team[c].name);
   
    /* scores for each problem */
    for (j = 0; j < num_probs; j++) {
      fprintf(out, "<td align=center>");
      if (team[c].attempt[j] > 0) {
	if (team[c].score[j] >= 0) {
	  /* got it right */
	  fprintf(out, "%d/%d", team[c].attempt[j], team[c].score[j]);
	} else if (team[c].score[j] == JUDGING) {
	  /* current attempt being judged */
	  fprintf(out, "%d/J", team[c].attempt[j]);
	} else {
	  /* never had it right */
	  fprintf(out, "%d/--", team[c].attempt[j]);
	}
      } else {
	/* never tried */
	fprintf(out, "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
      }
      fprintf(out, "</td>");
    }
    fprintf(out, "<td align=center>%d</td>", team[c].solved);
    fprintf(out, "<td align=center>%d</td>", team[c].total);
    fprintf(out, "<td align=center>%d</td>", team[c].rank);

    fprintf(out, "</tr>\n");
  }
  fprintf(out, "</table>\n");
  fprintf(out, "</body></html>\n");

  fclose(out);
}

static int is_blank(char *s)
{
  while (*s) {
    if (!isspace(*s)) {
      return 0;
    }
    s++;
  }
  return 1;
}

