#!/bin/sh
#
#
#  File:        adapted from v7judge.sh
#  Purpose:     used as scripts for judges to judge submissions
#  Author:      Howard Cheng <hchcheng@scg.math.uwaterloo.ca>
#
#  1:  name of file with extension (no path) ie.  A.java
#  2:  name of file with no extension
#  3:  language/compiler name
#
#  It is expected that the name of the file with no extension is the problem
#  letter name, and all input data files are of the form A.*.dat.
#
#  Examples:
#  sh v7judge.sh A.java A java
#  sh v7judge.sh A.c A cpp
#

#
# Time limits
#
CTIMELIMIT=5
JAVATIMELIMIT=40

#
# Some constants: the result codes are ordered so that the higher number
# ones override the lower number ones they occur for different files
#

RUNTIME_RETURN=1
TIMELIMIT_RETURN=2
SECURITY_RETURN=3

SUCCESS_RESULT=0
FORMAT_RESULT=1
INCORRECT_RESULT=2
RUNTIME_RESULT=3
TIMELIMIT_RESULT=4
SECURITY_RESULT=5
COMPILE_RESULT=6

#
# Name of output files
#
OUTFILE=output.$$
OUTFILE2=output2.$$
rm -f $OUTFILE $OUTFILE2

DATAHOME=/u/gvcormac/acm/pc2/data

#
# Viewer Command
#
VIEWER="xterm -e less "

echo "-- Date/time for run: " `date` >> $OUTFILE

if [ "$DISPLAY" = "" ] ; then
    echo "Note: DISPLAY environmental variable not set, bummer "
fi

#
# Under pc2 this should never occur, but if hand executed it may
#
if [ "$1" != "" ] ; then
	if [ ! -f "$1" ] ; then
		echo
	    echo "-- source file $1 does not exist   --" >> $OUTFILE
	    echo "Script halted, missing source file --" >> $OUTFILE
		echo 
		$VIEWER $OUTFILE
		exit 2
	fi
fi 

RESULT=$SUCCESS_RESULT
RUN=1

#
# Usage 
#
if [ "$1" = "" ] ; then

    echo "$0 filename basename langname "
    echo 
    echo Compiles filename and runs basename, output to a file named $OUTFILE 
    echo then displays that file using the following command: $VIEWER
    echo 
    echo Parameters:
    echo " filename - name of file with extension (no path) ie.  A.java"
    echo " basename - name of file with no extension ie. A"
    echo " langname - language/compiler name (Ex. java) "
    echo
    echo Example of usage: sh $0 hello.java hello java
    echo
    exit 2
#
# Compile the program
#
elif [ "$3" = "gcc" ] ; then

    CC=gcc
    CFLAGS="-O2 -Wall"
    LIBS="${HOME}/scripts/lockout.o -lm"

    # remove target executable
    rm -f $2

    echo "Compiling $1 using $CC "
    echo "======> Compiler listing for $1 using $3 <======" >> $OUTFILE

    ($CC $CFLAGS -o $2 $1 $LIBS 2>&1) >> $OUTFILE

    if [ ! -f ./$2 ] ; then 
      echo "Compile failed - program $2 not created"
      RUN=0
      RESULT=$COMPILE_RESULT
    fi

elif [ "$3" = "g++" ] ; then

    CC=g++
    CFLAGS="-O2 -Wall"
    LIBS="${HOME}/scripts/lockout.o -lm"

    # remove target executable
    rm -f $2

    echo "Compiling $1 using $CC "
    echo "======> Compiler listing for $1 using $3 <======" >> $OUTFILE

    ($CC $CFLAGS -o $2 $1 $LIBS 2>&1) >> $OUTFILE

    if [ ! -f ./$2 ] ; then 
      echo "Compile failed - program $2 not created"
      RUN=0
      RESULT=$COMPILE_RESULT
    fi

elif [ "$3" = "java" ] ; then

    # remove target executable
    rm -f $2.class

    echo "Compiling $1 using javac "

    echo "======> Compiler listing for $1 using $3 <======" >> $OUTFILE

    (javac -O $1 2>&1) >> $OUTFILE

    if [ $? -ne 0 ] ; then 
      echo "Compile failed - program $2.class not created"
      RESULT=$COMPILE_RESULT
      RUN=0
    fi

else

    echo "Error - Unknown language $3, contact Contest Staff "  
    echo "Error - Unknown language $3, contact Contest Staff "  > $OUTFILE
    RUN=0
fi

# now execute the program on each 
if [ $RUN -eq 1 ] ; then
  for testdata in ${DATAHOME}/$2.*.dat
  do
    TMPOUT=tmpout.$$
    rm -f $TMPOUT
    DIFFOUT=diffout.$$
    rm -f $DIFFOUT
    echo "-- Running $2 on $testdata --"
    echo "======> Output of $2 on $testdata <======" >> $OUTFILE2
    if [ "$3" = "java" ] ; then
      bash ${HOME}/scripts/runjava.sh $JAVATIMELIMIT $2 $testdata $TMPOUT
      STATUS=$?
    else
      ${HOME}/scripts/cwrapper $CTIMELIMIT $2 < $testdata > $TMPOUT
      STATUS=$?
      grep "The [A-Za-z0-9\_]*() function has been disallowed in this contest\." $TMPOUT
      if [ $? -eq 0 ] ; then
        STATUS=$SECURITY_RETURN
      fi
    fi
    if [ $STATUS -eq $TIMELIMIT_RETURN -a $TIMELIMIT_RESULT -gt $RESULT ] ; then
      RESULT=$TIMELIMIT_RESULT
    elif [ $STATUS -eq $SECURITY_RETURN -a $SECURITY_RESULT -gt $RESULT ] ; then
      RESULT=$SECURITY_RESULT
    elif [ $STATUS -eq $RUNTIME_RETURN -a $RUNTIME_RESULT -gt $RESULT ] ; then
      RESULT=$RUNTIME_RESULT
    fi;
    cat $TMPOUT >> $OUTFILE2
    echo "======> End of Output on $testdata <======" >> $OUTFILE2

    TESTBASE=`basename $testdata .dat`
    TESTBASE=${DATAHOME}/${TESTBASE}
    if [ -f ${TESTBASE}.cmd ] ; then
       echo "======> Judger's output for $testdata <======" >> $OUTFILE
       ${TESTBASE}.cmd  $TMPOUT < $TMPOUT >> $DIFFOUT
    else
       echo "======> DIFF of $testdata (> = judge's output) <======" >> $OUTFILE
       diff -H $TMPOUT ${TESTBASE}.diff >> $DIFFOUT
    fi
    if [ $? -ne 0 ] ; then
      echo "**** FAILED DIFF **** (return $?) " >> $OUTFILE
      cat $DIFFOUT >> $OUTFILE
      diff -HiBw $TMPOUT ${TESTBASE}.diff > /dev/null
      if [ $? -ne 0 ] ; then
        echo "**** INCORRECT OUTPUT ****" >> $OUTFILE
        if [ $INCORRECT_RESULT -gt $RESULT ] ; then
          RESULT=$INCORRECT_RESULT
        fi
      else
        echo "**** FORMAT ERROR ****" >> $OUTFILE
        if [ $FORMAT_RESULT -gt $RESULT ] ; then
          RESULT=$FORMAT_RESULT
        fi
      fi
    else
      cat $DIFFOUT >> $OUTFILE
      echo "SUCCESS" >> $OUTFILE
      echo "" >> $OUTFILE
    fi

    # don't waste any more time!
    if [ $RESULT -eq $TIMELIMIT_RESULT ] ; then
      break
    fi
  done
fi

echo "======> Suggested judgement: <======" >> $OUTFILE
case $RESULT in
    $SUCCESS_RESULT) echo "Correct" >> $OUTFILE;;
     $FORMAT_RESULT) echo "Presentation error" >> $OUTFILE;;
  $INCORRECT_RESULT) echo "Wrong answer" >> $OUTFILE;;
    $RUNTIME_RESULT) echo "Run-time error" >> $OUTFILE;;
   $SECURITY_RESULT) echo "Security violation" >> $OUTFILE;;
  $TIMELIMIT_RESULT) echo "Time limit exceeded" >> $OUTFILE;;
    $COMPILE_RESULT) echo "Compilation error" >> $OUTFILE;;
                  *) echo "Unknown RESULT!" >> $OUTFILE;;
esac



# view results
echo program done - starting viewer on result...

$VIEWER $OUTFILE &

if [ $RUN -eq 1 ] ; then
  echo "starting viewer on program's output"
  $VIEWER $OUTFILE2 &
fi
wait
rm -f $OUTFILE $OUTFILE2 $TMPOUT $DIFFOUT $2

#  end of script
